"""Interconversions between pairs, pairsql, pairsparquet"""
from pathlib import Path
from typing import *
import os
from parse import parse

def rename(name, pattern1, pattern2):
    """Reformat name"""
    parsed = parse(pattern1, name)
    if parsed is None:
        raise ValueError(f"Could not parse path {name} using pattern {pattern1}")
    named = parsed.named
    return pattern2.format(**named)

def autodetect_extension(path: Path | str, default: Tuple[str, str] = (None, None)) -> Tuple[str, str]:
    """Detect unique format-specifying extension from path
    
    Returns:
        Tuple[str]: format and extension identified in path 
    
    Raises:
        Exception if multiple file format extensions are detected
    """
    path = Path(path)
    suffixes = set(path.suffixes)

    autodetect = {
        "duckdb": [".duckdb", ".ddb", ".db", ".pairsduckdb"],
        "pairs": [".pairs"],
        "parquet": [".parquet", ".pq", ".pairsparquet"]
    }

    path_format = None
    format_extension = None
    for format, extensions in autodetect.items():
        detected_extension = set(extensions).intersection(suffixes)
        detected_extension = detected_extension.pop() if detected_extension else None

        repeated = path_format and detected_extension
        assert not repeated, f"Multiple file format extensions autodetected in {path}: {detected_extension} and {format_extension}."

        if detected_extension:
            path_format = format
            format_extension = detected_extension
    
    result = (path_format, format_extension) if path_format and format_extension else default

    assert result[0] and result[1], f"File format and extension not detected in {path} with suffixes {path.suffixes}. Options include {autodetect}. "

    return result


def walk_files(root_path: Path | str):
    """Walk through files in a directory"""
    for root, dirs, files in os.walk(root_path):
            if dirs:
                for dir in dirs:
                    yield from walk_files(root_path / dir)
            for file in files:
                yield Path(root) / file

def reorganize(out_path: Path | str, in_pattern: str = None, out_pattern: str = None, in_format: str = None, out_format: str = None, unlink: bool = True):
    """Walk through directory files and convert file formats and file names.
    
    Context: helper function for PairsSQL.partition_by
    """
    from hich.pairs.pairssql import PairsSQL
    assert not out_pattern or not Path(out_pattern).is_relative_to(out_path), f"--out-pattern {out_pattern} generates files in a subdirectory of OUT_PATH {out_path}, which is not allowed. Try using a different base directory in --out-pattern."
    out_path = Path(out_path)
    for partition_file in walk_files(out_path):
        if out_format != "parquet" or (in_pattern and out_pattern):
            if in_pattern and out_pattern:
                destination_file = Path(rename(str(partition_file), in_pattern, out_pattern))
            else:
                destination_file = Path(partition_file)

            destination_file.parent.mkdir(parents=True, exist_ok=True)

            if out_format == "parquet" or (not out_format and autodetect_extension(destination_file) == "parquet"):
                partition_file.rename(destination_file)
            else:
                db = PairsSQL.open(partition_file, out_format)

                db.write(destination_file, out_format)
                if partition_file.absolute() != destination_file.absolute() and unlink:
                    partition_file.unlink()


import click
import hictkpy
from hich.matrix.hicrep import HiCRepCaller
import itertools
from concurrent.futures import ProcessPoolExecutor
from pathlib import Path
import smart_open_with_pbgzip
from smart_open import smart_open

@click.group
def matrix():
    pass

@matrix.command
@click.option("--resolution", "-r", "resolutions", type=int, multiple=True, 
              help = "Contact matrix resolutions")
@click.option("--h", "-h", multiple=True, type=int, default = [1], 
              help = "Values of hicrep smoothing (h) parameters")
@click.option("--d-bp-max", "--dBPMax", "-m", multiple=True, type = int, default = [-1], 
              help = "Values of hicrep dBPMax parameter")
@click.option("--b-downsample", "--bDownSample", "-d", multiple=True, type=bool, default = [False], 
              help = "Values of hicrep bDownSample parameter")
@click.option("--normalization", "-n", "normalizations", multiple=True, type=str, default = None,
              help = "Contact matrix normalizations. If unspecified, uses all available; set --raw to include un-normalized with all the other normalizations. Use 'NONE' to call only on raw contact matrix.")
@click.option("--raw", is_flag = True, default = False, 
              help = "Call HiCRep without normalization (normalization = 'NONE') in addition to any other normalizations specified.")
@click.option("--one-norm", is_flag = True, default = False, help = "If multiple normalizations are present, only use one of them.")
@click.option("--chrom", "-c", "chroms", multiple=True, type=str, default=None, help = "The set of chromosomes to use. If unspecified, SCC scores are called on the subset of chromosomes shared by all contact matrices.")
@click.option("--skip-chrom", "-s", "skip_chroms", multiple=True, type=str, default=None, help = "Exclude these chromosomes, even if specified by --chrom")
@click.option("--region", "-g", multiple = True, type = str, default = None, help = "UCSC genome region to compare, i.e. chr1:10000000-11000000")
@click.option("--bed", "-b", multiple = True, type = str, default = None, help = "BED file defining a partition over the genome with HiCRep called on each region")
@click.option("--partition", "-p", multiple = True, type = int, default = None, help = "Partition selected chromosomes into uniform regions of this size")
@click.option("--n_proc", type=int, default=None, help = "Number of parallel processes to use to call SCC scores. By default, uses all available processors.")
@click.argument("out_path", type=str)
@click.argument("in_paths", type=str, nargs = -1)
def hicrep(resolutions, h, d_bp_max, b_downsample, normalizations, raw, one_norm, chroms, skip_chroms, region, bed, partition, n_proc, out_path, in_paths):
    """Call HiCRep SCC scores on combinations of contact matrices and parameterizations
    
    \b
    OUT_PATH: Output filename for results. The caller parameters are stored along with the resulting SCC score in the order returned.
    IN_PATHS: Two or more .hic or .cool/.mcool files (does not work on .scool files)

    \b
    Examples:

    \b
    Call SCC scores on all chromosomes shared by all contact matrices at 10kb and 100kb resolution using h values of 0, 1 and 2, scoring results in scc.txt:
        hich matrix hicrep -r 10000 -r 100000 -h 0 -h 1 -h 2 scc.txt mx1.mcool mx2.mcool mx3.mcool
    """
    # Get common chromosomes and normalizations for all files
    in_paths = set([Path(in_path).absolute() for in_path in in_paths])
    
    # Validate that we have at least two files and that all files exist
    assert all([it.exists() for it in in_paths]), f"Some input contact matrices do not exist on disk: {[it for it in in_paths if not it.exists()]}" 
    assert len(in_paths) >= 2, f"At least two contact matrices are required, but only {len(in_paths)} was suppled. IN_PATHS was {in_paths}."
    in_paths = [str(it) for it in in_paths]

    # Update chromosomes and normalizations
    chroms = set(chroms)
    normalizations = set(normalizations)

    if not chroms or not normalizations:
        # Get chromosomes and normalizations common to all datasets if not specified by the user
        update_chroms = None
        update_normalizations = None
        for path, resolution in itertools.product(in_paths, resolutions):
            file = hictkpy.File(path, resolution)
            if not chroms:
                new_chroms = set(file.chromosomes().keys())
                update_chroms = new_chroms if update_chroms is None else update_chroms.intersection(new_chroms)
            if not normalizations:
                new_normalizations = set(file.avail_normalizations())
                update_normalizations = new_normalizations if new_normalizations is None else new_normalizations.intersection(new_normalizations)
        chroms = update_chroms if not chroms else chroms
        normalizations = update_normalizations if not normalizations else normalizations

    # Add directive to use un-normalized matrix if requested
    if raw:
        normalizations.add("NONE")

    # Select a single normalization if requested
    if one_norm:
        chroms = set(list(chroms)[0])

    # Remove skipped chromosomes
    for chrom in skip_chroms:
        if chrom in chroms:
            chroms.discard(chrom)

    # Make sure we still have at least one chromosome and normalization
    assert chroms, f"No shared chromosomes amongst input matrices {in_paths}"
    assert normalizations, f"No shared normalizations amongst input matrices {in_paths}"

    # Create all individual combinations of parameters
    paths = itertools.combinations_with_replacement(in_paths, r=2)
    params = itertools.product(paths, chroms, normalizations, resolutions, h, d_bp_max, b_downsample)
    callers = [HiCRepCaller(*caller[0], *caller[1:]) for caller in params]

    # Call HiCRep SCC scores and write as TSV output
    f = smart_open(out_path, "wt")

    with ProcessPoolExecutor(max_workers=n_proc) as ppe:
        header_written = False
        for result in ppe.map(HiCRepCaller.call_scc, callers):
            if not header_written:
                header = "\t".join(result.as_dict().keys()) + "\n"
                f.write(header)
            row = "\t".join([str(val) for val in result.as_dict().values()]) + "\n"
            f.write(row)
    f.close()


""".pairssql is a DuckDB version of 4DN .pairs format"""
import smart_open_with_pbgzip
from smart_open import smart_open
import duckdb
import pyarrow
from hich.pairs.convert import *
from hich.pairs.header import * 
from dataclasses import dataclass
from parse import parse
import itertools
import io

@dataclass
class PairsSQL:
    path: Path = None
    _conn: duckdb.DuckDBPyConnection = None
    memory_limit: str = None

    def set_memory_limit(self, limit: str | None):
        self.conn.execute("SET memory_limit = $limit", {"limit": limit})


    def from_pairs(self, in_path: Path, out_path: Path | str = ":memory:"):
        self.connect(out_path)
        
        header = PairsHeader(in_path)
        
        
        self.path = out_path
        self.conn.execute(
f"""
DROP TABLE IF EXISTS pairs;
DROP TABLE IF EXISTS metadata;

CREATE TABLE pairs AS
SELECT * FROM read_csv($in_path, names=$names, comment='#', sample_size=-1);
""",
        {"in_path": str(in_path), "names":header.columns}
        )

        self.add_metadata(header)
        return self
    
    def from_duckdb(self, in_path: Path | str, out_path: Path | str = ":memory:"):
        self.connect(out_path)
        if in_path != out_path:
            self.conn.execute(
f"""
ATTACH '{in_path}' AS db1 (READ_ONLY);
ATTACH '{out_path}' AS db2;
DROP TABLE IF EXISTS pairs;
DROP TABLE IF EXISTS metadata;
COPY FROM DATABASE db1 TO db2;
DETACH db1;
USE db2;
"""
            )
        return self

    def write_duckdb(self, out_path: Path | str):
        self.conn.execute(
f"""
ATTACH '{out_path}' AS write_duckdb;
DROP TABLE IF EXISTS write_duckdb.pairs;
DROP TABLE IF EXISTS write_duckdb.metadata;
COPY FROM DATABASE {self.current_database} TO write_duckdb;
DETACH write_duckdb;
"""
        )
    
    def from_parquet(self, in_path: Path | str, out_path: Path | str = ":memory:"):
        self.connect(out_path)
        self.conn.execute(
f"""
CREATE TABLE pairs AS SELECT * FROM '{in_path}';
"""
        )
        
        metadata = self.conn.execute(f"SELECT * FROM parquet_kv_metadata('{in_path}')").fetchall()
        header_text = [row[2].decode('utf-8') for row in metadata if row[1].decode('utf-8') == "header"][0]
        header = PairsHeader(header_text, from_text = True, columns = self.columns, replace_columns = True)
        self.add_metadata(header)
        return self

    def add_metadata(self, header: PairsHeader):
        self.conn.execute(
f"""
CREATE TABLE IF NOT EXISTS metadata (
    table_name VARCHAR,
    header VARCHAR
);

DELETE FROM metadata WHERE table_name = 'pairs';

INSERT INTO metadata VALUES ('pairs', '{header.non_columns_text}');
"""
        )

    def write_pairs(self, out_path: Path | str, vector_multiple = 1000):
        handle = smart_open(out_path, "w")

        handle.write(self.header.text)
        query = self.conn.execute("SELECT * FROM pairs ORDER BY chrom1, chrom2, pos1, pos2")

        if vector_multiple is None:
            query.df().to_csv(handle, sep="\t", header=False, index=False, compression=None)
        else:
            while (chunk := query.fetch_df_chunk(vector_multiple)) is not None:
                if chunk.empty:
                    break
                else:
                    chunk.to_csv(handle, sep="\t", header=False, index=False, compression=None)
    

    def write_parquet(self, out_path: Path | str):
        self.conn.execute(
f"""
COPY (SELECT * FROM pairs)
TO '{out_path}'
(
    FORMAT PARQUET,
    KV_METADATA {{
        header: '{self.header.text}'
    }}
);
"""
        )

    def write(
            self,
            out_path: Path | str,
            out_format: str = "autodetect"
    ) -> duckdb.DuckDBPyConnection:
        """Write to pairs, duckdb, or parquet"""

        out_path = Path(out_path)

        if not out_format or out_format == "autodetect":
            out_format, _ = autodetect_extension(out_path)

        if out_format == "duckdb":
            self.write_duckdb(out_path)
        elif out_format == "parquet":
            self.write_parquet(out_path)
        elif out_format == "pairs":
            self.write_pairs(out_path)
        else:
            raise Exception(f"Format {out_format} not recognized. Should be one of duckdb, parquet, or pairs.")

    def partition_by(self, out_path: Path | str, partition_by: List[str] | str):
        partition_by = [partition_by] if isinstance(partition_by, str) else partition_by
        partition_by_str = ", ".join(partition_by)
        self.conn.execute(
f"""
COPY pairs TO '{out_path}' (
    FORMAT PARQUET, 
    PARTITION_BY ({partition_by_str}), 
    OVERWRITE_OR_IGNORE,
    KV_METADATA {{
        header: '{self.header.non_columns_text}'
    }}
)
"""
        )

    def iter_chroms(self, vector_multiple: int = None):
        """Yield Pandas dataframes in chunks in order of (chrom1, chrom2)
    
        Arguments:
            vector_multiple: The number of rows returned in each chunk is the vector size (2048 by default) * vector_multiple (1 by default).
        """
        combinations = self.conn.execute("SELECT DISTINCT chrom1, chrom2 FROM pairs ORDER BY chrom1, chrom2").pl()

        for chrom1, chrom2 in combinations.iter_rows():
            query = self.conn.execute(
                """
                SELECT * FROM pairs
                WHERE
                    chrom1 = $chrom1 
                    AND chrom2 = $chrom2
                ORDER BY chrom1, chrom2, pos1, pos2
                """,
                {"chrom1": chrom1, "chrom2": chrom2}
                            )
            if vector_multiple is None:
                yield query.df()
            else:
                while (chunk := query.fetch_df_chunk(vector_multiple)) is not None:
                    if chunk.empty:
                        break
                    else:
                        yield chunk
    

    @property
    def header(self) -> PairsHeader:
        header_text = self.conn.execute("SELECT * FROM metadata").fetchone()[1]
        return PairsHeader(header_text, from_text = True, columns = self.columns, replace_columns = True)

    @property
    def columns(self) -> List[str]:
        return [col[1] for col in self.conn.execute("PRAGMA table_info(pairs)").fetchall()]

    @property
    def conn(self) -> duckdb.DuckDBPyConnection:
        if self._conn is None:
            self._conn = duckdb.connect(self.path)
            if self.memory_limit:
                self._conn.set_memory_limit()
        return self._conn
    
    @property
    def current_database(self) -> str:
        return self.conn.execute("SELECT current_database()").fetchone()[0]
    
    def connect(self, out_path: Path | str = ":memory:") -> duckdb.DuckDBPyConnection:
        try:
            self._conn.close()
        except:
            pass
        self._conn = duckdb.connect(out_path)
        return self._conn

    @classmethod    
    def open(
            cls, 
            in_path: Path | str,
            in_format: str = "autodetect"
    ) -> "PairsSQL":
        """Open pairs, pairs, or parquet as PairsSQL object"""
        in_path = Path(in_path)
        if not in_format or in_format == "autodetect":
            in_format, _ = autodetect_extension(in_path)

        if in_format == "duckdb":
            result = PairsSQL().from_duckdb(in_path)
        elif in_format == "parquet":
            result = PairsSQL().from_parquet(in_path)
        elif in_format == "pairs":
            result = PairsSQL().from_pairs(in_path)
        else:
            raise Exception(f"Format {in_format} not recognized. Should be one of duckdb, parquet, or pairs.")


        return result


import click
import hich.fasta.digest_re
import polars
import smart_open_with_pbgzip
from collections import Counter
from smart_open import smart_open
from Bio import SeqIO


@click.group()
def fasta():
    pass

@fasta.command
@click.argument("in_path")
@click.argument("bp", type=int)
def gc(in_path, bp):
    """Compute fraction of GC content on uniform-size partition over the genome

    IN_PATH: Path to a fasta file (may be compressed)
    BP: Size in bp of partition blocks (the last block on each chromosome may be shorter)

    \b
    Example:
    Compute GC on a 100kb partition
        hich fasta gc hg38.fasta.gz 100000 > hg38_gc_100kb.bed
    """
    try:
        fasta = smart_open(in_path, "rt")
    except Exception as e:
        print(f"Unable to open {in_path}.")
        raise e

    try:
        for record in SeqIO.parse(fasta, "fasta"):
            for start in range(0, len(record.seq), bp):
                end = min(len(record.seq), start + bp)
                counts = Counter(record[start:end])
                g = counts.get("G", 0) + counts.get("g", 0)
                c = counts.get("C", 0) + counts.get("c", 0)
                t = counts.get("T", 0) + counts.get("t", 0)
                a = counts.get("A", 0) + counts.get("a", 0)
                total = g + c + t + a
                gc_content = (g+c)/total if total else ""
                print(f"{record.id}\t{start}\t{end}\t{gc_content}\n", end="")
    except Exception as e:
        print(f"Exception raised trying to parse {in_path} as fasta file to compute GC content.")
        raise e

@fasta.command
@click.argument("in_path")
@click.argument("out_path")
@click.argument("enzymes_and_protocols", nargs=-1)
def re_fragments(in_path, out_path, enzymes_and_protocols):
    """Create restriction fragment index in BED format

    \b
    IN_PATH: FASTA file to digest
    OUT_PATH: Name of BED file to output digest breakpoints
    ENZYMES_AND_PROTOCOLS: One or more space-separated REBASE restriction enzyme names (case sensitive), or protocols or Hi-C kit names (case-insensitive).

    \b
    Examples:
        hich fasta digest-re hg38.fasta hg38_digest.bed DpnII DdeI
        hich fasta digest-re hg38.fasta hg38_digest.bed hic3.0
        hich fasta digest-re hg38.fasta hg38_digest.bed Arima

    \b        
    In addition to any combination of REBASE enzymes, you can invoke hich fasta digest re with any of the following protocol and kit names:

    \b
    ENZYMES_AND_PROTOCOLS arguments and corresponding enzymes:
    "HiC 3.0" --> Enzymes: ['DpnII', 'DdeI']
    "Hi-C 3.0" --> Enzymes: ['DpnII', 'DdeI']
    "Arima Genome-Wide HiC+" --> Enzymes: ['DpnII', 'HinfI']
    "Arima" --> Enzymes: ['DpnII', 'HinfI']
    "Phase Proximo 2021+ Plant" --> Enzymes: ['DpnII']
    "Phase Plant" --> Enzymes: ['DpnII']
    "Phase Proximo 2021+ Animal" --> Enzymes: ['DpnII']
    "Phase Animal" --> Enzymes: ['DpnII']
    "Phase Proximo 2021+ Microbiome" --> Enzymes: ['Sau3AI', 'MluCI']
    "Phase Microbiome" --> Enzymes: ['Sau3AI', 'MluCI']
    "Phase Proximo 2021+ Human" --> Enzymes: ['DpnII']
    "Phase Human" --> Enzymes: ['DpnII']
    "Phase Proximo 2021+ Fungal" --> Enzymes: ['DpnII']
    "Phase Fungal" --> Enzymes: ['DpnII']

    \b
    Notes:
    1. Protocol names are case-insensitive, so you can call hich fasta digest_re [IN_PATH] [OUT_PATH] 'hic 3.0'.
    2. Use quotes for protocol names with spaces.
    3. The enzyme list is filtered for uniqueness so repeated enzyme names don't affect results.
    """
    handle = smart_open(out_path, "wt")
    for cutsites in hich.fasta.digest_re.digest_re(in_path, enzymes_and_protocols):
        for row in cutsites.iter_rows():
            outputs = [str(it) for it in row]
            text = "\t".join(outputs) + "\n"
            handle.write(text)



from hicrep.hicrep import sccByDiag
from hicrep.utils import *
import scipy.sparse as sp
import hictkpy
import numpy as np
import warnings
from pathlib import Path
from dataclasses import dataclass
import dataclasses
from typing import *

@dataclass
class HiCRepCaller:
    path1: Path | str
    path2: Path | str
    chromosome: str
    normalization: str
    resolution: int
    h: int
    d_bp_max: int
    b_downsample: int
    scc: float = None

    @property
    def files(self) -> Tuple[hictkpy.File, hictkpy.File]:
        """Return hictkpy.File handles for the two paths"""
        return (
            hictkpy.File(self.path1, self.resolution),
            hictkpy.File(self.path2, self.resolution)
        )

    @classmethod
    def call_scc(cls, caller: "HiCRepCaller"):
        file1, file2 = caller.files
        try:
            caller.scc = hicrep_scc_chrom(file1, file2, caller.chromosome, caller. normalization, caller.h, caller.d_bp_max, caller.b_downsample)
        except Exception as e:
            caller.scc = None
            warnings.warn(f"Exception on {caller}:\n{e}")

        return caller

    def as_dict(self):
        return {field.name: getattr(self, field.name) for field in dataclasses.fields(self)}


def hicrep_scc_chrom(file1: hictkpy.File, file2: hictkpy.File, chrom: str, normalization: str | None,
              h: int, dBPMax: int, bDownSample: bool) -> float:
    """Compute hicrep SCC score between two input hictkpy Files on a specific chromosome

    Args:
        file1: `hictkpy.File` Handle to the first file in the comparison
        file2: `hictkpy.File` Handle to the second file in the comparison
        chrom: `str`: The chromosome name to compare
        h: `int` Half-size of the mean filter used to smooth the
        input matrics
        dBPMax `int` Only include contacts that are at most this genomic
        distance (bp) away
        bDownSample: `bool` Down sample the input with more contacts
        to the same number of contacts as in the other input

    Returns:
        `float` scc score for the chromosome
    """
    # !Warning: this method has no specific unit test as of 2024/10/20 - Ben Skubi
    
    # Get resolution for each contact matrix
    resolution1 = file1.resolution()
    resolution2 = file2.resolution()

    # Input validation:
    # Same resolution
    # Both have the chromosome
    # Same number of bins per chromosome
    error_preamble = f"Input files {file1.path()} and {file2.path()}"

    assert resolution1 == resolution2,\
        f"{error_preamble} have different resolutions {resolution1} and {resolution2}"
    assert chrom in file1.chromosomes(), f"{file1.path()} does not contain chromosome {chrom}"
    assert chrom in file2.chromosomes(), f"{file2.path()} does not contain chromosome {chrom}"

    bins1 = file1.bins().query(f"chrom == '{chrom}'")
    bins2 = file2.bins().query(f"chrom == '{chrom}'")
    resolution = resolution1

    if resolution is None:
        # sometimes bin size can be None, e.g., input file has
        # non-uniform size bins.
        assert np.all(bins1[:] == bins2[:]),\
            f"Input contact matrix files don't have a unique bin size most likely "\
            f"because non-uniform bin size was used and the bins are defined "\
            f"differently for the two input contact matrix files"
        # In that case, use the median bin size
        resolution = int(np.median((bins1[:]["end"] - bins1[:]["start"]).values))
        warnings.warn(f"Input contact matrix files don't have a unique bin size most "\
                      f"likely because non-uniform bin size was used. HicRep "\
                      f"will use median bin size from the first contact matrix file "\
                      f"to determine maximal diagonal index to include", RuntimeWarning)

    assert len(bins1) == len(bins2),\
        f"{error_preamble} have different number of bins for chromosome {chrom} {len(bins1)} and {len(bins2)}"
    
    # Validate dBPMax
    if dBPMax == -1 or dBPMax is None:
        # this is the exclusive upper bound
        dMax = resolution
    else:
        dMax = dBPMax // resolution + 1

    assert dMax > 1, f"Input dBPmax is smaller than binSize"

    normalization = "NONE" if normalization is None else normalization

    # Get pixels
    p1 = file1.fetch(chrom).to_df()
    p2 = file2.fetch(chrom).to_df()
    file1.fetch()

    m1 = pixels2Coo(p1, bins1)
    m2 = pixels2Coo(p2, bins2)

    del p1
    del p2
    
    # get the total number of contacts as normalizing constant
    
    n1 = np.sum(m1.data[~np.isnan(m1.data)])
    n2 = np.sum(m2.data[~np.isnan(m2.data)])

    # Input validation on shape of chromosome matrix
    assert m1.size > 0, f"{error_preamble}, contact matrix 1 of chromosome {chrom} is empty"
    assert m1.shape[0] == m1.shape[1], f"{error_preamble}, contact matrix 1 of chromosome {chrom} is not square"

    assert m2.size > 0, f"{error_preamble}, contact matrix 2 of chromosome {chrom} is empty"
    assert m2.shape[0] == m2.shape[1], f"{error_preamble}, contact matrix 2 of chromosome {chrom} is not square"

    assert m1.shape == m2.shape, f"{error_preamble}, contact matrices of chromosome {chrom} have different input shape"

    return computeSCC(m1, m2, n1, n2, dMax, h, bDownSample)

def computeSCC(mS1: sp.coo_matrix,
               mS2: sp.coo_matrix,
               n1: float,
               n2: float,
               dMax: int,
               h: float,
               bDownSample: bool) -> float:

    """computeSCC 
    """
    # Compute scc score
    nDiags = mS1.shape[0] if dMax < 0 else min(dMax, mS1.shape[0])

    # remove major diagonal and all the diagonals >= nDiags
    # to save computation time
    m1 = trimDiags(mS1, nDiags, False)
    m2 = trimDiags(mS2, nDiags, False)
    del mS1
    del mS2

    if bDownSample:
        # do downsampling
        size1 = m1.sum()
        size2 = m2.sum()
        if size1 > size2:
            m1 = resample(m1, size2).astype(float)
        elif size2 > size1:
            m2 = resample(m2, size1).astype(float)
    else:
        # just normalize by total contacts
        m1 = m1.astype(float) / n1
        m2 = m2.astype(float) / n2

    if h > 0:
        # apply smoothing
        m1 = meanFilterSparse(m1, h)
        m2 = meanFilterSparse(m2, h)

    return sccByDiag(m1, m2, nDiags)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['hich', 'hich.commands', 'hich.fasta', 'hich.matrix', 'hich.pairs']

package_data = \
{'': ['*']}

entry_points = \
{'console_scripts': ['hich = hich.__main__:hich']}

setup_kwargs = {
    'name': 'hich',
    'version': '0.1.8',
    'description': 'CLI tools for Hi-C data processing',
    'long_description': None,
    'author': 'Ben Skubi',
    'author_email': 'skubi@ohsu.edu',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'entry_points': entry_points,
    'python_requires': '>=3.11,<4.0',
}


setup(**setup_kwargs)
